<?php
class ebrowser_vfs_local {
private static $Information;
private static $Entries;
private function path ($Path) {
	if ($Path && $Path[0] == '/') $Path = '.'.$Path;
	else if (!$Path || (substr ($Path, 0, 2) != './' && $Path[0] != '/')) $Path = './'.$Path;
	return (str_replace ('//', '/', $Path));
	}
public function __construct ($Path) {}
public function refresh () {
	clearstatcache ();
	self::$Information = self::$Entries = array ();
	}
public function type () {
	return ('local');
	}
public function prefix () {
	return ('');
	}
public function size ($Path) {
	$Path = $this->path ($Path);
	if (!is_dir ($Path)) return (filesize ($Path));
	$Size = 0;
	$Entries = $this->entries ($Path);
	for ($i = 0, $c = count ($Entries); $i < $c; ++$i) $Size += $this->size ($Entries[$i]);
	return ($Size);
	}
public function create ($Path, $Directory = 0) {
	$Path = $this->path ($Path);
	if ($Directory) return (mkdir ($Path));
	else return (touch ($Path));
	}
public function save ($Path, $Contents) {
	$Path = $this->path ($Path);
	return (file_put_contents ($Path, $Contents));
	}
public function contents ($Path) {
	$Path = $this->path ($Path);
	return (file_get_contents ($Path));
	}
public function exists ($Path) {
	$Path = $this->path ($Path);
	return (file_exists ($Path));
	}
public function chmod ($Path, $Mode) {
	$Path = $this->path ($Path);
	return (chmod ($Path, $Mode));
	}
public function delete ($Path) {
	$Path = $this->path ($Path);
	if (!is_dir ($Path)) {
		if (unlink ($Path)) {
			e_location_update ($Path);
			}
		return (0);
		}
	if (rmdir ($Path)) {
		e_location_update ($Path);
		return (1);
		}
	$this->chmod (substr ($Path, -1), 0777);
	$Entries = $this->entries ($Path);
	for ($i = 0, $c = count ($Entries); $i < $c; ++$i) $this->delete ($Entries[$i]);
	return ($this->delete ($Path));
	}
public function rename ($From, $To) {
	$From = $this->path ($From);
	$To = $this->path ($To);
	return (rename ($From, $To));
	}
public function symlink ($From, $To) {
	$From = realpath ($this->path ($From));
	$To = realpath ($this->path ($To));
	return (symlink ($From, $To));
	}
public function copy ($From, $To) {
	return (copy ($this->path ($From), $this->path ($To)));
	}
public function entries ($Path, $Exclude = '') {
	$Path = $this->path ($Path);
	if (isset (self::$Entries[$Path])) return (self::$Entries[$Path]);
	$Array = array ();
	if (!is_dir ($Path) || !is_readable ($Path)) return ($Array);
	$RealPath = realpath ($Path);
	if ($Path && substr ($Path, -1) != '/') $Path.= '/';
	if ($Handle = opendir ($Path)) {
		while ($Entry = readdir ($Handle)) {
			if ($Entry == '.' || $Entry == '..' || ($Exclude && preg_match ($Exclude, $Entry)) || ($RealPath.'/'.$Entry == $GLOBALS['BrowserPath'] && !defined ('EBROWSER_ADMIN'))) continue;
			$Array[] = $Path.$Entry;
			}
		closedir ($Handle);
		}
	self::$Entries[$Path] = &$Array;
	return ($Array);
	}
public function information ($Path) {
	$Path = $this->path ($Path);
	if (isset (self::$Information[$Path])) return (self::$Information[$Path]);
	if (!file_exists ($Path) && !is_link ($Path)) return (0);
	$MIME = $MetaData = '';
	$Stat = lstat ($Path);
	if (!$Stat) return (0);
	$IsDir = is_dir ($Path);
	$IsBroken = !is_readable ($Path);
	if ($IsDir && substr ($Path, -1) != '/') $Path.= '/';
	if ($IsDir) $Stat['size'] = count ($this->entries ($Path));
	else {
		if (function_exists ('finfo_file') && !$IsBroken) {
			$FileInfo = finfo_open (FILEINFO_MIME | FILEINFO_SYMLINK);
			$MIME = finfo_file ($FileInfo, realpath ($Path));
			finfo_close ($FileInfo);
			$FileInfo = finfo_open (FILEINFO_NONE);
			$MIMEDescription = finfo_file ($FileInfo, realpath ($Path));
			finfo_close ($FileInfo);
			}
		}
	$Address = '';
	$IsLink = 0;
	if (is_link ($IsDir?substr ($Path, 0, -1):$Path)) {
		$IsLink = 1;
		$Address = readlink ($IsDir?substr ($Path, 0, -1):$Path);
		}
	if (!$IsDir) {
		$Extension = explode ('.', basename ($IsLink?$Address:$Path));
		$Extension = strtoupper (end ($Extension));
		}
	else $Extension = '';
	$FileType = ($IsDir?array ('directory'):e_file_type ($Extension, $MIME));
	if ($Extension == 'URL') {
		$URL = parse_ini_file ($Path);
		if (isset ($URL['URL'])) $Address = htmlspecialchars ($URL['URL']);
		}
	if ($Extension == 'DESKTOP') {
		$URL = parse_ini_file ($Path);
		if (isset ($URL['URL'])) {
			$Address = htmlspecialchars ($URL['URL']);
			$FileType = array ('url');
			}
		}
	if (defined ('EBROWSER_ADMIN')) {
		if (function_exists ('posix_getgrgid')) {
			$Group = posix_getgrgid ($Stat['gid']);
			$Owner = posix_getpwuid ($Stat['uid']);
			}
		else $Group = $Owner = array ('name' => '');
		$Permissions = ($IsBroken?0:fileperms ($Path));
		}
	if (!isset ($FileType[1])) $FileType[1] = $Extension;
	if (in_array ($FileType[1], array ('JPEG', 'TIFF')) && extension_loaded ('exif')) {
		if ($MetaData = exif_read_data ($Path)) $MetaData = (isset ($MetaData['Make'])?'Make: <em>'.$MetaData['Make'].'</em><br />
':'').(isset ($MetaData['Model'])?'Model: <em>'.$MetaData['Model'].'</em><br />
':'').(isset ($MetaData['FileDateTime'])?'Time: <em>'.date ('Y.m.d H:i:s', $MetaData['FileDateTime']).'</em><br />
':'').(isset ($MetaData['XResolution'])?'XResolution: <em>'.$MetaData['XResolution'].'</em><br />
':'').(isset ($MetaData['YResolution'])?'YResolution: <em>'.$MetaData['YResolution'].'</em><br />
':'').(isset ($MetaData['ExposureTime'])?'ExposureTime: <em>'.$MetaData['ExposureTime'].'</em><br />
':'').(isset ($MetaData['ExposureProgram'])?'ExposureProgram: <em>'.$MetaData['ExposureProgram'].'</em><br />
':'').(isset ($MetaData['FNumber'])?'FNumber: <em>'.$MetaData['FNumber'].'</em><br />
':'').(isset ($MetaData['ISOSpeedRatings'])?'ISOSpeedRatings: <em>'.$MetaData['ISOSpeedRatings'].'</em><br />
':'').(isset ($MetaData['ApertureValue'])?'ApertureValue: <em>'.$MetaData['ApertureValue'].'</em><br />
':'').(isset ($MetaData['Flash'])?'Flash: <em>'.$MetaData['Flash'].'</em><br />
':'').(isset ($MetaData['Orientation'])?'Orientation: <em>'.$MetaData['Orientation'].'</em><br />
':'');
		else $MetaData = '';
		}
	if ($FileType[1] == 'RPM' && extension_loaded ('RPMReader')) {
		$RPM = rpm_open ($Path);
		if (!$RPM) $IsBroken = 1;
		else $MetaData = 'RPM name: <em>'.rpm_get_tag ($RPM, RPMREADER_NAME).'<br />
Version: <em>'.rpm_get_tag ($RPM, RPMREADER_VERSION).'</em><br />
Release: <em>'.rpm_get_tag ($RPM, RPMREADER_RELEASE).'</em><br />
Architecture: <em>'.rpm_get_tag ($RPM, RPMREADER_ARCH).'</em><br />
';
		rpm_close ($RPM);
		}
/*	if ($FileType[1] == 'MP3' && extension_loaded ('id3')) {
		$MetaData = id3_get_tag ($Path, ID3_BEST);
		if ($MetaData) $MetaData = 'Title: <em>'.htmlspecialchars ($MetaData['title']).'</em> (<em>'.htmlspecialchars ($MetaData['album']).'</em>)<br />
Artist: <em>'.htmlspecialchars ($MetaData['artist']).'</em><br />
Genre: <em>'.id3_get_genre_name ($MetaData['genre']).'</em><br />
';
		}*/
	$Array = e_file_information (array (
	'access' => $Stat['atime'],
	'address' => $Address,
	'broken' => $IsBroken,
	'group' => (defined ('EBROWSER_ADMIN')?$Group['name']:''),
	'link' => $IsLink,
	'metadata' => $MetaData,
	'mime' => ($MIME?$MIMEDescription.' ('.$MIME.')':''),
	'owner' => (defined ('EBROWSER_ADMIN')?$Owner['name']:''),
	'path' => htmlspecialchars (substr ($Path, 1)),
	'permissions' => (defined ('EBROWSER_ADMIN')?$Permissions:''),
	'readable' => is_readable ($Path),
	'size' => $Stat['size'],
	'time' => $Stat['mtime'],
	'writeable' => is_writeable ($Path)
	));
	if ($Path == './') {
		$Array['name'] = e_i18n ('Main directory');
		$Array['icon'] = e_icon_path ('folder-home');
		}
	self::$Information[$Path] = &$Array;
	return ($Array);
	}
}
?>